/*
EquLines (Equalize Lines) filter
Version 0.2 plugin for Avisynth 2.5
Copyright (c)2003 Alexander G. Balakhnin aka Fizick
bag@hotmail.ru

This filter equalize total luminosity in pairs of even and odd lines.
The filter preserves all details (no smoothing or deinterlacing).

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   
    
Apllications.
    
The simple filter is useful for removing of inter-line difference
from captured (telecined) 8mm films (kino, movie).
The well-known Antiflicker filter (by Donald Graft) make a good job
in frames luminosity equalizing, but may to leave some inter-field differences,
which are revealed after restoring of a progressive frames sequence
(with Telecide and Decimate filters).
The EquLines filter must apllied after them all.

Works only with Avisynth C Interface (avithynth_c.dll) from Kevin Atkinson.
Based on his example filter "invert_luma_in_place"

Using:

Avisource("file.avi")
Loadplugin("avisynth_c.dll")
LoadCplugin("EquLines.dll")
EquLines(deltamax=120)
  
Optional parameter Deltamax is the maximum of luminisity change (default=255)
Filter needs YUY2 or YV12

  Source may be compiled with GNU GCC 3.3 (MSYS 1.0, MinGW)
  Tested with Avisynth 2.5.2
*/

#include "avisynth_c.h"

static inline int min(int x, int y) {return x < y ? x : y;}
static inline int max(int x, int y) {return x > y ? x : y;}
static inline int abs(int x) {return x > 0 ? x : -x;}

//
// EquLines
//

 
AVS_VideoFrame * equ_lines_get_frame (AVS_FilterInfo * p, int n)
{
  AVS_VideoFrame * frame;
  int row_size, height, pitch, height2;
  BYTE * data;
  int y, x;
  long luma1, luma2;
  int delta_luma;
  int delta_max = 1;
  int width, drow, r;

  frame = avs_get_frame(p->child, n);
  avs_make_writable(p->env, &frame);
  row_size = avs_get_row_size(frame); // number of bytes per width
  height = avs_get_height(frame);
  pitch = avs_get_pitch(frame);      // number of bytes per line in memory
  data  = avs_get_write_ptr(frame);
  height2 = height/2; 
  delta_max = (int)p->user_data;

// Look every pair of lines
  for (y = 0; y != height2; ++y) {
    luma1=0;
    luma2=0;
    if (avs_is_yuy2(&p->vi)) 
      drow = 2;     // number of byte per pixel for YUY2
    else          
      drow =1;      //        YU12
   
    width = row_size/drow;
// calculate summary luma for line 1 and 2
      for (x = 0; x < width; x ++) {
        r = x*drow;
        luma1 = luma1 + data[r];
        luma2 = luma2 + data[r+pitch];
      }
// calculate delta (correction) of lumas
      delta_luma = (luma2-luma1)/(width*2);
      if ( abs(delta_luma) < delta_max ) 
// equalize medium lumas for line 1 and 2
        for (x = 0; x < width; x ++) {
        r = x*drow;
          data[r] = max( min( (data[r] + delta_luma), 255), 0);
          data[r+pitch] =max( min( (data[r+pitch] - delta_luma), 255), 0);;
        }
// Get next two lines
    data += pitch ;
    data += pitch ;
  }
  return frame;
}

AVS_Value equ_lines (AVS_ScriptEnvironment * env, 
				AVS_Value args, void * dg)
{
  AVS_Value v;
  AVS_FilterInfo * fi;

  AVS_Value tmp;
  int deltamax;

  AVS_Clip * new_clip = avs_new_c_filter(env, &fi, avs_array_elt(args,0), 1);
  if (avs_is_yuy2(&fi->vi) || avs_is_yv12(&fi->vi)) {
//   get first argument
    tmp = avs_array_elt(args, 1);
     if (avs_defined(tmp)) 
        deltamax = avs_as_int(tmp);
     else
        deltamax = 255 ;  // set default value 255 if no argument
     
    fi->user_data  =(void *)deltamax;

    fi->get_frame = equ_lines_get_frame;
    v = avs_new_value_clip(new_clip);
  }
  else {
    v = avs_new_value_error("EquLines: Video Must be YUY2 or YV12!");
  }
  avs_release_clip(new_clip);
  return v;
}


//
// Filter registration
//

const char * avisynth_c_plugin_init(AVS_ScriptEnvironment * env)
{
  avs_add_function(env, "EquLines", "c[deltamax]i", equ_lines, 0);
  return "";
}


